# flake8: noqa
from typing import Optional

import os

api_key: Optional[str] = None
client_id: Optional[str] = None
client_secret: Optional[str] = None
api_production: str = "https://api.pingen.com"
auth_production: str = "https://identity.pingen.com"
api_staging: str = "https://api-staging.pingen.com"
auth_staging: str = "https://identity-staging.pingen.com"
request_timeout: int = 20

# OAuth
from pingen2sdk.oauth import OAuth

# PingenResponse
from pingen2sdk.response import PingenResponse

# PingenError
from pingen2sdk.error import PingenError

# APIRequestor
from pingen2sdk.api import APIRequestor

# API Resources
from pingen2sdk.api_resources import *

# PingenWebhook
from pingen2sdk.webhook_event import WebhookEvent

# PingenWebhook
from pingen2sdk.incoming_webhook import IncomingWebhook



import json

from typing import Dict, Optional


class PingenError(Exception):
    _message: Optional[str]
    json_body: Optional[object]
    status_code: Optional[int]
    headers: Optional[Dict[str, str]]
    request_id: Optional[str]

    def __init__(
        self,
        message: Optional[str] = None,
        body: Optional[str] = None,
        status_code: Optional[int] = None,
        headers: Optional[Dict[str, str]] = None,
    ):
        super(PingenError, self).__init__(message)

        self._message = message

        if bool(body):
            self.json_body = json.loads(body)

        self.headers = headers or {}
        self.status_code = status_code
        self.request_id = self.headers.get("x-request-id", None)


class AuthenticationError(PingenError):
    pass


class WebhookSignatureException(Exception):
    _message: Optional[str]

    def __init__(self, message=""):
        self.message = message
        super().__init__(self.message)



import json

from typing import Mapping, Optional


class PingenResponse(object):
    body: str
    status_code: int
    headers: Mapping[str, str]
    data: Optional[object] = None

    def __init__(self, body: str, status_code: int, headers: Mapping[str, str]):
        self.body = body

        if bool(body):
            self.data = json.loads(body)

        self.status_code = status_code
        self.headers = headers

    @property
    def request_id(self) -> Optional[str]:
        return self.headers.get("x-request-id", None)



import pingen2sdk
from typing import Tuple


class FileUpload(object):
    def __init__(
        self,
        api_requestor: pingen2sdk.APIRequestor,
    ):
        self.api_requestor = api_requestor

    def request_file_upload(self) -> Tuple[str, str]:
        response = self.api_requestor.perform_get_request(
            "/file-upload",
        )

        return (
            response.data["data"]["attributes"]["url"],
            response.data["data"]["attributes"]["url_signature"],
        )

    def put_file(
        self,
        path_to_file: str,
        file_url: str,
    ):
        file = open(path_to_file, "rb")

        self.api_requestor.perform_put_request(file_url, file)

        file.close()



import pingen2sdk
import responses


class TestOrganisations(object):
    @staticmethod
    def _construct_resource():
        access_token = "test_access_token"

        return pingen2sdk.Organisations(access_token, True)

    @responses.activate
    def test_get_details(self):
        organisation_id = "orgxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxx1"
        url = "%s/organisations/%s" % (pingen2sdk.api_staging, organisation_id)

        organisations = self._construct_resource()

        responses.get(
            url,
            headers={
                "Content-Type": "application/vnd.api+json",
                "X-Request-Id": "requestx-xxxx-xxxx-xxxx-xxxxxxxxxxx1",
            },
            json={
                "data": {
                    "id": organisation_id,
                    "type": "organisations",
                    "attributes": {
                        "name": "ACME GmbH",
                        "status": "active",
                        "plan": "free",
                        "billing_mode": "prepaid",
                        "billing_currency": "CHF",
                        "billing_balance": 11.23,
                        "missing_credits": 0,
                        "default_country": "CH",
                        "edition": "string",
                        "default_address_position": "left",
                        "default_sender_address": "ACME GmbH | Strasse 3 | 8000 Zürich",
                        "data_retention_addresses": 18,
                        "data_retention_pdf": 12,
                        "color": "#0758FF",
                        "created_at": "2020-11-19T09:42:48+0100",
                        "updated_at": "2020-11-19T09:42:48+0100",
                    },
                    "relationships": {
                        "associations": {
                            "links": {
                                "related": {"href": "string", "meta": {"count": 0}}
                            }
                        }
                    },
                    "links": {"self": "string"},
                    "meta": {"abilities": {"self": {"manage": "ok"}}},
                },
                "included": [{}],
            },
            status=200,
        )

        response = organisations.get_details(organisation_id)

        assert response.data["data"]["id"] == organisation_id
        assert response.data["data"]["type"] == "organisations"
        assert response.status_code == 200

    @responses.activate
    def test_get_collection(self):
        organisation_id = "orgxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxx11"
        url = "%s/organisations" % pingen2sdk.api_staging

        organisations = self._construct_resource()

        responses.get(
            url,
            headers={
                "Content-Type": "application/vnd.api+json",
                "X-Request-Id": "requestx-xxxx-xxxx-xxxx-xxxxxxxxxxx1",
            },
            json={
                "data": [
                    {
                        "id": organisation_id,
                        "type": "organisations",
                        "attributes": {
                            "name": "ACME GmbH",
                            "status": "active",
                            "plan": "free",
                            "billing_mode": "prepaid",
                            "billing_currency": "CHF",
                            "billing_balance": 11.23,
                            "missing_credits": 0,
                            "default_country": "CH",
                            "edition": "string",
                            "default_address_position": "left",
                            "default_sender_address": "ACME GmbH | Strasse 3 | 8000 Zürich",
                            "data_retention_addresses": 18,
                            "data_retention_pdf": 12,
                            "color": "#0758FF",
                            "created_at": "2020-11-19T09:42:48+0100",
                            "updated_at": "2020-11-19T09:42:48+0100",
                        },
                        "relationships": {
                            "associations": {
                                "links": {
                                    "related": {"href": "string", "meta": {"count": 0}}
                                }
                            }
                        },
                        "links": {"self": "string"},
                    }
                ],
                "included": [{}],
                "links": {
                    "first": "string",
                    "last": "string",
                    "prev": "string",
                    "next": "string",
                    "self": "string",
                },
                "meta": {
                    "current_page": 0,
                    "last_page": 0,
                    "per_page": 0,
                    "from": 0,
                    "to": 0,
                    "total": 0,
                },
            },
            status=200,
        )

        response = organisations.get_collection()

        assert response.data["data"][0]["id"] == organisation_id
        assert response.data["data"][0]["type"] == "organisations"
        assert response.data["data"][0]["attributes"]["name"] == "ACME GmbH"
        assert response.status_code == 200



import requests
import pingen2sdk

from io import IOBase, BytesIO
from typing import Any, Dict, Mapping, Optional, cast
from requests.models import PreparedRequest


def _response_interpreter(
    body: Dict,
    status_code: int,
    headers: Mapping[str, str],
) -> pingen2sdk.PingenResponse:
    if 200 <= status_code < 310:
        return pingen2sdk.PingenResponse(body, status_code, headers)
    else:
        raise pingen2sdk.PingenError(None, body, status_code, headers)


class APIRequestor(object):
    access_token: str
    api_base: str
    user_agent: str = "PINGEN.SDK.PYTHON"

    def __init__(self, access_token: str, use_staging: Optional[bool] = False):
        if use_staging is False:
            self.api_base = pingen2sdk.api_production
        else:
            self.api_base = pingen2sdk.api_staging

        self.access_token = access_token

    def perform_get_request(
        self,
        url: str,
        params: Optional[Mapping[str, Any]] = None,
        supplied_headers: Optional[Mapping[str, str]] = None,
    ) -> pingen2sdk.PingenResponse:
        r = requests.get(
            self.prepare_path(url, params),
            headers=self.request_headers(supplied_headers),
            timeout=pingen2sdk.request_timeout,
        )

        return _response_interpreter(r.text, r.status_code, r.headers)

    @staticmethod
    def perform_put_request(
        url: str,
        file: BytesIO,
    ):
        requests.put(url, data=file)

    def perform_post_request(
        self,
        url: str,
        payload: str,
        supplied_headers: Optional[Mapping[str, str]] = None,
    ) -> pingen2sdk.PingenResponse:
        r = requests.post(
            self.prepare_path(url),
            payload,
            headers=self.request_headers(supplied_headers),
            timeout=pingen2sdk.request_timeout,
        )

        return _response_interpreter(r.text, r.status_code, r.headers)

    def perform_patch_request(
        self,
        url: str,
        payload: str,
        supplied_headers: Optional[Mapping[str, str]] = None,
    ) -> pingen2sdk.PingenResponse:
        r = requests.patch(
            self.prepare_path(url),
            payload,
            headers=self.request_headers(supplied_headers),
            timeout=pingen2sdk.request_timeout,
        )

        return _response_interpreter(r.text, r.status_code, r.headers)

    def perform_cancel_request(self, url: str) -> pingen2sdk.PingenResponse:
        r = requests.patch(
            self.prepare_path(url),
            headers=self.request_headers(),
            timeout=pingen2sdk.request_timeout,
        )

        return _response_interpreter(r.text, r.status_code, r.headers)

    def perform_delete_request(self, url: str) -> pingen2sdk.PingenResponse:
        r = requests.delete(
            self.prepare_path(url),
            headers=self.request_headers(),
            timeout=pingen2sdk.request_timeout,
        )

        return _response_interpreter(r.text, r.status_code, r.headers)

    def perform_stream_request(
        self,
        url: str,
    ) -> IOBase:
        response = requests.get(
            self.prepare_path(url),
            headers=self.request_headers(),
            timeout=pingen2sdk.request_timeout,
            stream=True,
        )

        return cast(IOBase, response.text)

    def prepare_path(
        self,
        url: str,
        params: Optional[Mapping[str, Any]] = None,
    ) -> str:
        path = "%s%s" % (self.api_base, url)

        request = PreparedRequest()
        request.prepare_url(path, params)

        return request.url

    def request_headers(
        self,
        supplied_headers: Optional[Mapping[str, str]] = None,
    ) -> Mapping[str, str]:
        supplied_headers_dict: Optional[Dict[str, str]] = (
            dict(supplied_headers) if supplied_headers is not None else None
        )

        headers = {
            "User-Agent": self.user_agent,
            "Authorization": "Bearer {}".format(self.access_token),
            "Content-Type": "application/vnd.api+json",
            "Accept": "application/vnd.api+json",
        }

        if supplied_headers_dict is not None:
            for key, value in supplied_headers_dict.items():
                headers[key] = value

        return headers



import pingen2sdk
import responses


class TestEbills(object):
    @staticmethod
    def _construct_resource():
        access_token = "test_access_token"
        organisation_id = "testxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxx1"

        return pingen2sdk.Ebills(organisation_id, access_token)

    @staticmethod
    def _get_expected_payload(ebill_id: str):
        return {
            "data": {
                "id": ebill_id,
                "type": "ebills",
                "attributes": {
                    "status": "string",
                    "file_original_name": "lorem.pdf",
                    "file_pages": 2,
                    "recipient_identifier": "41100010014282213",
                    "invoice_number": "Invoice 8051",
                    "invoice_date": "2025-10-01",
                    "invoice_due_date": "2025-10-30",
                    "invoice_value": 1.25,
                    "invoice_currency": "CHF",
                    "price_value": 1.25,
                    "price_currency": "CHF",
                    "source": "api",
                    "submitted_at": "2021-11-19T09:42:48+0100",
                    "created_at": "2020-11-19T09:42:48+0100",
                    "updated_at": "2020-11-19T09:42:48+0100",
                },
                "relationships": {
                    "organisation": {
                        "links": {"related": "string"},
                        "data": {
                            "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                            "type": "organisations",
                        },
                    },
                    "events": {
                        "links": {
                            "related": {"href": "string", "meta": {"count": 0}}
                        }
                    },
                },
                "links": {"self": "string"},
            },
            "included": [{}],
        }

    @staticmethod
    def mock_file_upload_request():
        responses.get(
            "%s/file-upload" % (pingen2sdk.api_production,),
            headers={
                "Content-Type": "application/vnd.api+json",
                "X-Request-Id": "requestx-xxxx-xxxx-xxxx-xxxxxxxxxxx1",
            },
            json={
                "data": {
                    "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                    "type": "file_uploads",
                    "attributes": {
                        "url": "https://s3.example/bucket/filename?signer=url",
                        "url_signature": "$2y$10$BLOzVbYTXrh4LZbSYNVf7eEDrc58vvQ9PRVZABqV/9WS1eqIcm3M",
                        "expires_at": "2020-11-19T09:42:48+0100",
                    },
                    "links": {"self": "string"},
                }
            },
        )

        responses.put("https://s3.example/bucket/filename?signer=url", status=201)

    @responses.activate
    def test_get_ebill(self):
        ebill_id = "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxx1"
        url = "%s/organisations/testxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxx1/deliveries/ebills/%s" % (
            pingen2sdk.api_production,
            ebill_id,
        )

        ebills = self._construct_resource()

        responses.get(
            url,
            headers={
                "Content-Type": "application/vnd.api+json",
                "X-Request-Id": "requestx-xxxx-xxxx-xxxx-xxxxxxxxxxx1",
            },
            json=self._get_expected_payload("xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxx1"),
            status=200,
        )

        response = ebills.get_details(
            ebill_id, None, {"Content-Type": "application/vnd.api+json"}
        )

        assert response.data["data"]["id"] == ebill_id
        assert response.status_code == 200
        assert response.headers == {
            "Content-Type": "application/vnd.api+json",
            "X-Request-Id": "requestx-xxxx-xxxx-xxxx-xxxxxxxxxxx1",
        }

    @responses.activate
    def test_get_collection(self):
        url = (
            "%s/organisations/testxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxx1/deliveries/ebills"
            % pingen2sdk.api_production
        )

        ebills = self._construct_resource()

        responses.get(
            url,
            headers={
                "Content-Type": "application/vnd.api+json",
                "X-Request-Id": "requestx-xxxx-xxxx-xxxx-xxxxxxxxxxx1",
            },
            json={
                "data": [
                    {
                        "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                        "type": "ebills",
                        "attributes": {
                            "status": "string",
                            "file_original_name": "lorem.pdf",
                            "file_pages": 2,
                            "recipient_identifier": "41100010014282213",
                            "invoice_number": "Invoice 8051",
                            "invoice_date": "2025-10-01",
                            "invoice_due_date": "2025-10-30",
                            "invoice_value": 1.25,
                            "invoice_currency": "CHF",
                            "price_value": 1.25,
                            "price_currency": "CHF",
                            "source": "api",
                            "submitted_at": "2021-11-19T09:42:48+0100",
                            "created_at": "2020-11-19T09:42:48+0100",
                            "updated_at": "2020-11-19T09:42:48+0100",
                        },
                        "relationships": {
                            "organisation": {
                                "links": {"related": "string"},
                                "data": {
                                    "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                                    "type": "organisations",
                                },
                            },
                            "events": {
                                "links": {
                                    "related": {"href": "string", "meta": {"count": 0}}
                                }
                            },
                        },
                        "links": {"self": "string"},
                    }
                ],
                "included": [{}],
                "links": {
                    "first": "string",
                    "last": "string",
                    "prev": "string",
                    "next": "string",
                    "self": "string",
                },
                "meta": {
                    "current_page": 0,
                    "last_page": 0,
                    "per_page": 0,
                    "from": 0,
                    "to": 0,
                    "total": 0,
                },
            },
            status=200,
        )

        response = ebills.get_collection()

        assert response.data["data"][0]["id"] == "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx"
        assert response.status_code == 200
        assert response.headers == {
            "Content-Type": "application/vnd.api+json",
            "X-Request-Id": "requestx-xxxx-xxxx-xxxx-xxxxxxxxxxx1",
        }

    @responses.activate
    def test_create_ebill(self):
        url = "%s/organisations/testxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxx1/deliveries/ebills" % (
            pingen2sdk.api_production,
        )

        ebills = self._construct_resource()

        responses.post(
            url,
            headers={
                "Content-Type": "application/vnd.api+json",
                "X-Request-Id": "requestx-xxxx-xxxx-xxxx-xxxxxxxxxxx2",
            },
            json=self._get_expected_payload("xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxx11"),
            status=201,
        )

        response = ebills.create(
            "https://s3.example/bucket/filename?signer=url",
            "$2y$10$BLOzVbYTXrh4LZbSYNVf7eEDrc58vvQ9PRVZABqV/9WS1eqIcm3M",
            "lorem.pdf",
            True,
            None,
            {
                "preset": {
                    "data": {
                        "id": "xxxxxxxx-xxxx-xxxx-0000-xxxxxxxxx111",
                        "type": "presets",
                    }
                }
            },
        )

        assert response.data["data"]["id"] == "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxx11"
        assert response.status_code == 201
        assert response.headers == {
            "Content-Type": "application/vnd.api+json",
            "X-Request-Id": "requestx-xxxx-xxxx-xxxx-xxxxxxxxxxx2",
        }

    @responses.activate
    def test_upload_and_create_ebill(self):
        url = "%s/organisations/testxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxx1/deliveries/ebills" % (
            pingen2sdk.api_production,
        )

        ebills = self._construct_resource()
        self.mock_file_upload_request()

        responses.post(
            url,
            headers={
                "Content-Type": "application/vnd.api+json",
                "X-Request-Id": "requestx-xxxx-xxxx-xxxx-xxxxxxxxxxx3",
            },
            json=self._get_expected_payload("xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxx111"),
            status=201,
        )

        response = ebills.upload_and_create(
            "tests/api_resources/files/lorem.pdf",
            "lorem.pdf",
            False,
            {
                "invoice_number": "Invoice 8051",
                "invoice_date": "2025-10-16",
                "invoice_due_date": "2025-10-30",
                "recipient_identifier": "41100010014282213",
            },
        )

        assert response.data["data"]["id"] == "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxx111"
        assert response.status_code == 201
        assert response.headers == {
            "Content-Type": "application/vnd.api+json",
            "X-Request-Id": "requestx-xxxx-xxxx-xxxx-xxxxxxxxxxx3",
        }



import pingen2sdk
import responses


class TestBatches(object):
    @staticmethod
    def _construct_resource():
        access_token = "test_access_token"
        organisation_id = "testxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxx1"

        return pingen2sdk.Batches(organisation_id, access_token)

    @staticmethod
    def _get_expected_payload(batch_id: str):
        return {
            "data": {
                "id": batch_id,
                "type": "batches",
                "attributes": {
                    "name": "Monthly Invoicing August 2022",
                    "icon": "campaign",
                    "status": "string",
                    "file_original_name": "lorem.pdf",
                    "letter_count": 2,
                    "address_position": "left",
                    "print_mode": "simplex",
                    "print_spectrum": "color",
                    "price_currency": "CHF",
                    "price_value": 1.25,
                    "source": "api",
                    "submitted_at": "2021-11-19T09:42:48+0100",
                    "created_at": "2020-11-19T09:42:48+0100",
                    "updated_at": "2020-11-19T09:42:48+0100",
                },
                "relationships": {
                    "organisation": {
                        "links": {"related": "string"},
                        "data": {
                            "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                            "type": "organisations",
                        },
                    },
                    "events": {
                        "links": {"related": {"href": "string", "meta": {"count": 0}}}
                    },
                },
                "links": {"self": "string"},
                "meta": {
                    "abilities": {
                        "self": {
                            "cancel": "ok",
                            "delete": "ok",
                            "submit": "ok",
                            "edit": "ok",
                            "change-window-position": "ok",
                            "add-attachment": "ok",
                        }
                    }
                },
            },
            "included": [{}],
        }

    @staticmethod
    def mock_file_upload_request():
        responses.get(
            "%s/file-upload" % (pingen2sdk.api_production,),
            headers={
                "Content-Type": "application/vnd.api+json",
                "X-Request-Id": "requestx-xxxx-xxxx-xxxx-xxxxxxxxxxx1",
            },
            json={
                "data": {
                    "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                    "type": "file_uploads",
                    "attributes": {
                        "url": "https://s3.example/bucket/filename?signer=url",
                        "url_signature": "$2y$10$BLOzVbYTXrh4LZbSYNVf7eEDrc58vvQ9PRVZABqV/9WS1eqIcm3M",
                        "expires_at": "2020-11-19T09:42:48+0100",
                    },
                    "links": {"self": "string"},
                }
            },
        )

        responses.put("https://s3.example/bucket/filename?signer=url", status=201)

    @responses.activate
    def test_get_batch(self):
        batch_id = "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxx1"
        url = "%s/organisations/testxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxx1/batches/%s" % (
            pingen2sdk.api_production,
            batch_id,
        )

        batches = self._construct_resource()

        responses.get(
            url,
            headers={
                "Content-Type": "application/vnd.api+json",
                "X-Request-Id": "requestx-xxxx-xxxx-xxxx-xxxxxxxxxxx1",
            },
            json=self._get_expected_payload("xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxx1"),
            status=200,
        )

        response = batches.get_details(
            batch_id, None, {"Content-Type": "application/vnd.api+json"}
        )

        assert response.data["data"]["id"] == batch_id
        assert response.data["data"]["type"] == "batches"
        assert response.status_code == 200

    @responses.activate
    def test_get_collection(self):
        url = (
            "%s/organisations/testxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxx1/batches"
            % pingen2sdk.api_production
        )

        batches = self._construct_resource()

        responses.get(
            url,
            headers={
                "Content-Type": "application/vnd.api+json",
                "X-Request-Id": "requestx-xxxx-xxxx-xxxx-xxxxxxxxxxx1",
            },
            json={
                "data": [
                    {
                        "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                        "type": "batches",
                        "attributes": {
                            "status": "string",
                            "file_original_name": "lorem.pdf",
                            "file_pages": 2,
                            "address": "Hans Meier\nExample street 4\n8000 Zürich\nSwitzerland",
                            "address_position": "left",
                            "country": "CH",
                            "delivery_product": "fast",
                            "print_mode": "simplex",
                            "print_spectrum": "color",
                            "price_currency": "CHF",
                            "price_value": 1.25,
                            "paper_types": ["normal", "qr"],
                            "fonts": [
                                {"name": "Helvetica", "is_embedded": True},
                                {"name": "Helvetica-Bold", "is_embedded": False},
                            ],
                            "source": "api",
                            "tracking_number": "98.1234.11",
                            "sender_address": "ACME GmbH | Strasse 3 | 8000 Zürich",
                            "submitted_at": "2021-11-19T09:42:48+0100",
                            "created_at": "2020-11-19T09:42:48+0100",
                            "updated_at": "2020-11-19T09:42:48+0100",
                        },
                        "relationships": {
                            "organisation": {
                                "links": {"related": "string"},
                                "data": {
                                    "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                                    "type": "organisations",
                                },
                            },
                            "events": {
                                "links": {
                                    "related": {"href": "string", "meta": {"count": 0}}
                                }
                            },
                            "batch": {
                                "links": {"related": "string"},
                                "data": {
                                    "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                                    "type": "batches",
                                },
                            },
                        },
                        "links": {"self": "string"},
                    }
                ],
                "included": [{}],
                "links": {
                    "first": "string",
                    "last": "string",
                    "prev": "string",
                    "next": "string",
                    "self": "string",
                },
                "meta": {
                    "current_page": 0,
                    "last_page": 0,
                    "per_page": 0,
                    "from": 0,
                    "to": 0,
                    "total": 0,
                },
            },
            status=200,
        )

        response = batches.get_collection()

        assert response.data["data"][0]["id"] == "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx"
        assert response.data["data"][0]["type"] == "batches"
        assert response.status_code == 200

    @responses.activate
    def test_create_batch(self):
        url = "%s/organisations/testxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxx1/batches" % (
            pingen2sdk.api_production,
        )

        batches = self._construct_resource()

        responses.post(
            url,
            headers={
                "Content-Type": "application/vnd.api+json",
                "X-Request-Id": "requestx-xxxx-xxxx-xxxx-xxxxxxxxxxx2",
            },
            json=self._get_expected_payload("xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxx11"),
            status=201,
        )

        response = batches.create(
            "https://s3.example/bucket/filename?signer=url",
            "$2y$10$BLOzVbYTXrh4LZbSYNVf7eEDrc58vvQ9PRVZABqV/9WS1eqIcm3M",
            "lorem.pdf",
            "flash",
            "lorem.pdf",
            "left",
            "merge",
            "page",
            1,
            ",",
            "last_page",
        )

        assert response.data["data"]["id"] == "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxx11"
        assert response.status_code == 201
        assert response.headers == {
            "Content-Type": "application/vnd.api+json",
            "X-Request-Id": "requestx-xxxx-xxxx-xxxx-xxxxxxxxxxx2",
        }

    @responses.activate
    def test_upload_and_create_batch(self):
        url = "%s/organisations/testxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxx1/batches" % (
            pingen2sdk.api_production,
        )

        batches = self._construct_resource()
        self.mock_file_upload_request()

        responses.post(
            url,
            headers={
                "Content-Type": "application/vnd.api+json",
                "X-Request-Id": "requestx-xxxx-xxxx-xxxx-xxxxxxxxxxx3",
            },
            json=self._get_expected_payload("xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxx111"),
            status=201,
        )

        response = batches.upload_and_create(
            "tests/api_resources/files/lorem.pdf",
            "testing",
            "flash",
            "lorem.pdf",
            "left",
            "merge",
            "file",
        )

        assert response.data["data"]["id"] == "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxx111"
        assert response.status_code == 201
        assert response.headers == {
            "Content-Type": "application/vnd.api+json",
            "X-Request-Id": "requestx-xxxx-xxxx-xxxx-xxxxxxxxxxx3",
        }

    @responses.activate
    def test_send_batch(self):
        batch_id = "testsend-xxxx-xxxx-xxxx-xxxxxxxxxxx1"
        url = (
            "%s/organisations/testxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxx1/batches/%s/send"
            % (pingen2sdk.api_production, batch_id)
        )

        batches = self._construct_resource()

        responses.patch(
            url,
            headers={
                "Content-Type": "application/vnd.api+json",
                "X-Request-Id": "requestx-xxxx-xxxx-xxxx-xxxxxxxxx332",
            },
            json=self._get_expected_payload(batch_id),
            status=200,
        )

        response = batches.send(
            batch_id,
            [
                {"country": "CH", "delivery_product": "postag_a"},
                {"country": "DE", "delivery_product": "fast"},
            ],
            "simplex",
            "color",
        )

        assert response.data["data"]["id"] == batch_id
        assert response.status_code == 200
        assert response.headers == {
            "Content-Type": "application/vnd.api+json",
            "X-Request-Id": "requestx-xxxx-xxxx-xxxx-xxxxxxxxx332",
        }
        assert response.request_id == "requestx-xxxx-xxxx-xxxx-xxxxxxxxx332"

    @responses.activate
    def test_cancel_batch(self):
        batch_id = "testsend-xxxx-xxxx-xxxx-xxxxxxxxxxx1"
        url = (
            "%s/organisations/testxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxx1/batches/%s/cancel"
            % (pingen2sdk.api_production, batch_id)
        )

        batches = self._construct_resource()

        responses.patch(
            url,
            status=202,
        )

        response = batches.cancel(
            batch_id,
        )

        assert response.status_code == 202

    @responses.activate
    def test_delete_batch(self):
        batch_id = "testdelx-xxxx-xxxx-xxxx-xxxxxxxxxxx1"
        url = "%s/organisations/testxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxx1/batches/%s" % (
            pingen2sdk.api_production,
            batch_id,
        )

        batches = self._construct_resource()

        responses.delete(
            url,
            status=204,
        )

        response = batches.delete(
            batch_id,
        )

        assert response.status_code == 204

    @responses.activate
    def test_edit_batch(self):
        batch_id = "testedit-xxxx-xxxx-xxxx-xxxxxxxxx551"
        url = "%s/organisations/testxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxx1/batches/%s" % (
            pingen2sdk.api_production,
            batch_id,
        )

        batches = self._construct_resource()

        responses.patch(
            url,
            headers={
                "Content-Type": "application/vnd.api+json",
                "X-Request-Id": "requestx-xxxx-xxxx-xxxx-xxxxxxxxx552",
            },
            json=self._get_expected_payload(batch_id),
            status=200,
        )

        response = batches.edit(
            batch_id,
            list({"normal", "qr"}),
        )

        assert response.data["data"]["id"] == batch_id
        assert response.status_code == 200
        assert response.headers == {
            "Content-Type": "application/vnd.api+json",
            "X-Request-Id": "requestx-xxxx-xxxx-xxxx-xxxxxxxxx552",
        }

    @responses.activate
    def test_get_statistics(self):
        batch_id = "testxxxx-xxxx-xxxx-xxxx-xxxxxx321111"
        url = (
            "%s/organisations/testxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxx1/batches/%s/statistics"
            % (pingen2sdk.api_production, batch_id)
        )

        batches = self._construct_resource()

        responses.get(
            url,
            headers={
                "Content-Type": "application/vnd.api+json",
                "X-Request-Id": "requestx-xxxx-xxxx-xxxx-xxxxxxxx1332",
            },
            json={
                "data": {
                    "id": batch_id,
                    "type": "batch_statistics",
                    "attributes": {
                        "letter_validating": 2,
                        "letter_groups": [
                            {"name": "valid", "count": 3},
                            {"name": "not_available", "count": 1},
                        ],
                        "letter_countries": [
                            {"country": "CH", "count": 3},
                            {"country": "DE", "count": 1},
                        ],
                    },
                    "links": {"self": "string"},
                }
            },
            status=202,
        )

        response = batches.get_statistics(batch_id)

        assert response.data["data"]["id"] == batch_id
        assert response.data["data"]["type"] == "batch_statistics"



import pingen2sdk
import json

from typing import Any, Mapping, Optional


class Webhooks(object):
    api_requestor: pingen2sdk.APIRequestor

    def __init__(
        self,
        organisation_id: str,
        access_token: str,
        use_staging: bool = False,
    ):
        self.organisation_id = organisation_id
        self.api_requestor = pingen2sdk.APIRequestor(access_token, use_staging)

    def get_details(
        self,
        webhook_id: str,
        params: Optional[Mapping[str, Any]] = None,
        supplied_headers: Optional[Mapping[str, str]] = None,
    ) -> pingen2sdk.PingenResponse:
        return self.api_requestor.perform_get_request(
            "/organisations/%s/webhooks/%s" % (self.organisation_id, webhook_id),
            params,
            supplied_headers,
        )

    def get_collection(
        self,
        params: Optional[Mapping[str, Any]] = None,
        supplied_headers: Optional[Mapping[str, str]] = None,
    ) -> pingen2sdk.PingenResponse:
        return self.api_requestor.perform_get_request(
            "/organisations/%s/webhooks" % self.organisation_id,
            params,
            supplied_headers,
        )

    def create(
        self, event_category: str, url: str, signing_key: str
    ) -> pingen2sdk.PingenResponse:
        attributes = {
            "event_category": event_category,
            "url": url,
            "signing_key": signing_key,
        }

        return self.api_requestor.perform_post_request(
            "/organisations/%s/webhooks" % self.organisation_id,
            json.dumps({"data": {"type": "webhooks", "attributes": attributes}}),
        )

    def delete(
        self,
        webhook_id: str,
    ) -> pingen2sdk.PingenResponse:
        return self.api_requestor.perform_delete_request(
            "/organisations/%s/webhooks/%s" % (self.organisation_id, webhook_id),
        )



# flake8: noqa
from pingen2sdk.api_resources.letters import Letters as Letters
from pingen2sdk.api_resources.file_upload import FileUpload as FileUpload
from pingen2sdk.api_resources.letter_events import LetterEvents as LetterEvents
from pingen2sdk.api_resources.users import Users as Users
from pingen2sdk.api_resources.user_associations import (
    UserAssociations as UserAssociations,
)
from pingen2sdk.api_resources.organisations import Organisations as Organisations
from pingen2sdk.api_resources.batches import Batches as Batches
from pingen2sdk.api_resources.batch_events import BatchEvents as BatchEvents
from pingen2sdk.api_resources.webhooks import Webhooks as Webhooks
from pingen2sdk.api_resources.ebills import Ebills as Ebills
from pingen2sdk.api_resources.emails import Emails as Emails



import pingen2sdk
import json

from typing import Any, Mapping, Optional, Dict


class Ebills(object):
    def __init__(
        self,
        organisation_id: str,
        access_token: str,
        use_staging: bool = False,
    ):
        self.organisation_id = organisation_id
        self.api_requestor = pingen2sdk.APIRequestor(access_token, use_staging)

    def get_details(
        self,
        ebill_id: str,
        params: Optional[Mapping[str, Any]] = None,
        supplied_headers: Optional[Mapping[str, str]] = None,
    ) -> pingen2sdk.PingenResponse:
        return self.api_requestor.perform_get_request(
            "/organisations/%s/deliveries/ebills/%s" % (self.organisation_id, ebill_id),
            params,
            supplied_headers,
        )

    def get_collection(
        self,
        params: Optional[Mapping[str, Any]] = None,
        supplied_headers: Optional[Mapping[str, str]] = None,
    ) -> pingen2sdk.PingenResponse:
        return self.api_requestor.perform_get_request(
            "/organisations/%s/deliveries/ebills" % self.organisation_id,
            params,
            supplied_headers,
        )

    def upload_and_create(
        self,
        path_to_file: str,
        file_original_name: str,
        auto_send: False,
        meta_data: Optional[Dict] = None,
        relationships: Optional[Dict] = None,
    ) -> pingen2sdk.PingenResponse:
        file_upload = pingen2sdk.FileUpload(self.api_requestor)
        file_url, file_signature = file_upload.request_file_upload()
        file_upload.put_file(path_to_file, file_url)

        return self.create(
            file_url,
            file_signature,
            file_original_name,
            auto_send,
            meta_data,
            relationships,
        )

    def create(
        self,
        file_url: str,
        file_signature: str,
        file_original_name: str,
        auto_send: False,
        meta_data: Optional[Dict] = None,
        relationships: Optional[Dict] = None,
    ) -> pingen2sdk.PingenResponse:
        attributes = {
            "file_original_name": file_original_name,
            "file_url": file_url,
            "file_url_signature": file_signature,
            "auto_send": auto_send,
        }

        if meta_data is not None:
            attributes["meta_data"] = meta_data

        payload: Dict[str, Any] = {
            "data": {
                "type": "ebills",
                "attributes": attributes,
            }
        }

        if relationships is not None:
            payload["data"]["relationships"] = relationships

        return self.api_requestor.perform_post_request(
            "/organisations/%s/deliveries/ebills" % self.organisation_id,
            json.dumps(payload),
        )
